/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMSelection.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "Range.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomDOMSelection.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomRange.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDOMSelection.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateGdomRange.h"
#include "gdom/PrivateIntGdomDOMSelection.h"
#include "gdom/PrivateIntGdomNode.h"
#include "gdom/PrivateIntGdomRange.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::DOMSelection* obj)
{
    return WebCore::cacheDOMObject<WebCore::DOMSelection, GDOMDOMSelection>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMDOMSelection::GDOMDOMSelection(PassRefPtr<WebCore::DOMSelection> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMDOMSelection::~GDOMDOMSelection()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

void
gdom_dom_selection_collapse (GdomDOMSelection *thiz, GdomNode * node, glong index)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (node);
    WebCore::Node * _g_node = (gdom_node_get_private(node));
    g_return_if_fail (_g_node);
    WebCore::ExceptionCode ec = 0;
    item->collapse(_g_node, index, ec);

}

void
gdom_dom_selection_collapse_to_end (GdomDOMSelection *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    item->collapseToEnd();

}

void
gdom_dom_selection_collapse_to_start (GdomDOMSelection *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    item->collapseToStart();

}

void
gdom_dom_selection_delete_from_document (GdomDOMSelection *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    item->deleteFromDocument();

}

gboolean
gdom_dom_selection_contains_node (GdomDOMSelection *thiz, GdomNode * node, gboolean allow_partial)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (node, 0);
    WebCore::Node * _g_node = (gdom_node_get_private(node));
    g_return_val_if_fail (_g_node, 0);
    gboolean res = item->containsNode(_g_node, allow_partial);
    return res;

}

void
gdom_dom_selection_select_all_children (GdomDOMSelection *thiz, GdomNode * node)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (node);
    WebCore::Node * _g_node = (gdom_node_get_private(node));
    g_return_if_fail (_g_node);
    WebCore::ExceptionCode ec = 0;
    item->selectAllChildren(_g_node, ec);

}

void
gdom_dom_selection_empty (GdomDOMSelection *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    item->empty();

}

void
gdom_dom_selection_set_base_and_extent (GdomDOMSelection *thiz, GdomNode * base_node, glong base_offset, GdomNode * extent_node, glong extent_offset)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (base_node);
    g_return_if_fail (extent_node);
    WebCore::Node * _g_base_node = (gdom_node_get_private(base_node));
    g_return_if_fail (_g_base_node);
    WebCore::Node * _g_extent_node = (gdom_node_get_private(extent_node));
    g_return_if_fail (_g_extent_node);
    WebCore::ExceptionCode ec = 0;
    item->setBaseAndExtent(_g_base_node, base_offset, _g_extent_node, extent_offset, ec);

}

void
gdom_dom_selection_set_position (GdomDOMSelection *thiz, GdomNode * node, glong offset)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (node);
    WebCore::Node * _g_node = (gdom_node_get_private(node));
    g_return_if_fail (_g_node);
    WebCore::ExceptionCode ec = 0;
    item->setPosition(_g_node, offset, ec);

}

void
gdom_dom_selection_modify (GdomDOMSelection *thiz, gchar * alter, gchar * direction, gchar * granularity)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (alter);
    g_return_if_fail (direction);
    g_return_if_fail (granularity);
    WebCore::String _g_alter = WebCore::String::fromUTF8(alter);
    WebCore::String _g_direction = WebCore::String::fromUTF8(direction);
    WebCore::String _g_granularity = WebCore::String::fromUTF8(granularity);
    item->modify(_g_alter, _g_direction, _g_granularity);

}

void
gdom_dom_selection_extend (GdomDOMSelection *thiz, GdomNode * node, glong offset)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (node);
    WebCore::Node * _g_node = (gdom_node_get_private(node));
    g_return_if_fail (_g_node);
    WebCore::ExceptionCode ec = 0;
    item->extend(_g_node, offset, ec);

}

GdomRange *
gdom_dom_selection_get_range_at (GdomDOMSelection *thiz, glong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Range> g_res = WTF::getPtr(item->getRangeAt(index, ec));
    GdomRange * res = static_cast<GdomRange *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_dom_selection_remove_all_ranges (GdomDOMSelection *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    item->removeAllRanges();

}

void
gdom_dom_selection_add_range (GdomDOMSelection *thiz, GdomRange * range)
{
    g_return_if_fail (thiz);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (range);
    WebCore::Range * _g_range = (gdom_range_get_private(range));
    g_return_if_fail (_g_range);
    item->addRange(_g_range);

}

gchar *
gdom_dom_selection_to_string (GdomDOMSelection *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMSelection * item = (gdom_dom_selection_get_private(thiz));
    g_return_val_if_fail (item, 0);
    gchar * res = _G_GET_UTF8(item->toString());
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "DOMSelection.h"
#include <gdom/GdomDOMSelection.h>
#include <gdom/PrivateGdomDOMSelection.h>
#include <gdom/PrivateIntGdomDOMSelection.h>
#include <gdom/GdomDOMSelectionprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomDOMSelection, gdom_dom_selection, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomDOMSelectionPrivate {
    class WebCore::DOMSelection *item;
};

#define GDOM_DOM_SELECTION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_DOM_SELECTION, GdomDOMSelectionPrivate))
*/
G_END_DECLS
WebCore::DOMSelection *
gdom_dom_selection_get_private(GdomDOMSelection *request)
{
    GDOMDOMSelection *res;
    res = static_cast<GDOMDOMSelection*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::DOMSelection*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_ANCHOR_NODE,
    PROP_ANCHOR_OFFSET,
    PROP_FOCUS_NODE,
    PROP_FOCUS_OFFSET,
    PROP_BASE_NODE,
    PROP_BASE_OFFSET,
    PROP_EXTENT_NODE,
    PROP_EXTENT_OFFSET,
    PROP_IS_COLLAPSED,
    PROP_TYPE,
    PROP_RANGE_COUNT,
};


static void gdom_dom_selection_finalize(GObject* object)
{
    /*GdomDOMSelection* request = GDOM_DOM_SELECTION(object);*/
    /*GdomDOMSelectionPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_dom_selection_parent_class)->finalize(object);
}

static void gdom_dom_selection__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomDOMSelection *self = GDOM_DOM_SELECTION(object);
    WebCore::DOMSelection * gself = (gdom_dom_selection_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_dom_selection__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomDOMSelection *self = GDOM_DOM_SELECTION(object);
    WebCore::DOMSelection * gself = (gdom_dom_selection_get_private(self));
    switch (prop_id) {
    case PROP_ANCHOR_NODE:
    {
         PassRefPtr<WebCore::Node> ptr = gself->anchorNode();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_ANCHOR_OFFSET:
    {
         g_value_set_long (value, gself->anchorOffset());
         break;
    }
    case PROP_FOCUS_NODE:
    {
         PassRefPtr<WebCore::Node> ptr = gself->focusNode();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_FOCUS_OFFSET:
    {
         g_value_set_long (value, gself->focusOffset());
         break;
    }
    case PROP_BASE_NODE:
    {
         PassRefPtr<WebCore::Node> ptr = gself->baseNode();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_BASE_OFFSET:
    {
         g_value_set_long (value, gself->baseOffset());
         break;
    }
    case PROP_EXTENT_NODE:
    {
         PassRefPtr<WebCore::Node> ptr = gself->extentNode();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_EXTENT_OFFSET:
    {
         g_value_set_long (value, gself->extentOffset());
         break;
    }
    case PROP_IS_COLLAPSED:
    {
         g_value_set_boolean (value, gself->isCollapsed());
         break;
    }
    case PROP_TYPE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->type()));
         break;
    }
    case PROP_RANGE_COUNT:
    {
         g_value_set_long (value, gself->rangeCount());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_dom_selection_class_init(GdomDOMSelectionClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_dom_selection_finalize;
    g_klass->set_property = gdom_dom_selection__set_property_fn;
    g_klass->get_property = gdom_dom_selection__get_property_fn;

    g_object_class_install_property (g_klass, PROP_ANCHOR_NODE,
            g_param_spec_object ("anchor_node", /* name */
                "dom_selection_anchor_node", /* short description */
                "read-only  GdomNode * DOMSelection.anchor_node", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ANCHOR_OFFSET,
            g_param_spec_long ("anchor_offset", /* name */
                "dom_selection_anchor_offset", /* short description */
                "read-only  glong DOMSelection.anchor_offset", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_FOCUS_NODE,
            g_param_spec_object ("focus_node", /* name */
                "dom_selection_focus_node", /* short description */
                "read-only  GdomNode * DOMSelection.focus_node", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_FOCUS_OFFSET,
            g_param_spec_long ("focus_offset", /* name */
                "dom_selection_focus_offset", /* short description */
                "read-only  glong DOMSelection.focus_offset", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_BASE_NODE,
            g_param_spec_object ("base_node", /* name */
                "dom_selection_base_node", /* short description */
                "read-only  GdomNode * DOMSelection.base_node", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_BASE_OFFSET,
            g_param_spec_long ("base_offset", /* name */
                "dom_selection_base_offset", /* short description */
                "read-only  glong DOMSelection.base_offset", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_EXTENT_NODE,
            g_param_spec_object ("extent_node", /* name */
                "dom_selection_extent_node", /* short description */
                "read-only  GdomNode * DOMSelection.extent_node", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_EXTENT_OFFSET,
            g_param_spec_long ("extent_offset", /* name */
                "dom_selection_extent_offset", /* short description */
                "read-only  glong DOMSelection.extent_offset", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_IS_COLLAPSED,
            g_param_spec_boolean ("is_collapsed", /* name */
                "dom_selection_is_collapsed", /* short description */
                "read-only  gboolean DOMSelection.is_collapsed", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_TYPE,
            g_param_spec_string ("type", /* name */
                "dom_selection_type", /* short description */
                "read-only  gchar * DOMSelection.type", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_RANGE_COUNT,
            g_param_spec_long ("range_count", /* name */
                "dom_selection_range_count", /* short description */
                "read-only  glong DOMSelection.range_count", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomDOMSelectionPrivate));*/
}

static void gdom_dom_selection_init(GdomDOMSelection* request)
{
    /*GdomDOMSelectionPrivate* priv = GDOM_DOM_SELECTION_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

